@@ -2,10 +2,9 @@ package Kelp::Response;
 
 use Kelp::Base 'Plack::Response';
 
-use Encode;
 use Carp;
 use Try::Tiny;
-use Scalar::Util;
+use Scalar::Util qw(blessed);
 
 attr -app => sub { croak "app is required" };
 attr rendered => 0;
@@ -19,24 +18,29 @@ sub new {
 }
 
 sub set_content_type {
-    $_[0]->content_type( $_[1] );
-    return $_[0];
+    my ( $self, $type ) = @_;
+    $self->content_type( $type );
+    return $self;
 }
 
 sub text {
-    $_[0]->set_content_type( 'text/plain; charset=' . $_[0]->app->charset );
+    my $self = shift;
+    return $self->set_content_type( 'text/plain; charset=' . $self->app->charset );
 }
 
 sub html {
-    $_[0]->set_content_type( 'text/html; charset=' . $_[0]->app->charset );
+    my $self = shift;
+    return $self->set_content_type( 'text/html; charset=' . $self->app->charset );
 }
 
 sub json {
-    $_[0]->set_content_type('application/json');
+    my $self = shift;
+    return $self->set_content_type('application/json');
 }
 
 sub xml {
-    $_[0]->set_content_type('application/xml');
+    my $self = shift;
+    return $self->set_content_type('application/xml');
 }
 
 sub finalize {
@@ -83,12 +87,12 @@ sub render {
         die "Data must be a reference" unless ref($body);
         my $json = $self->app->json;
         $body = $json->encode($body);
-        $body = encode($self->app->charset, $body) unless $json->get_utf8;
+        $body = $self->app->charset_encode( $body ) unless $json->get_utf8;
         $self->json if $guess_json;
         $self->body( $body );
     } else {
         $self->html if $guess_html;
-        $self->body( encode( $self->app->charset, $body ) );
+        $self->body( $self->app->charset_encode( $body ) );
     }
 
     $self->rendered(1);
@@ -173,7 +177,7 @@ sub render_500 {
     }
 
     # if render_500 gets blessed object as error stringify it
-    $error = ref $error if Scalar::Util::blessed $error;
+    $error = ref $error if blessed $error;
 
     return $self->set_code(500)->render($error);
 }
@@ -453,3 +457,4 @@ module.
     }
 
 =cut
+
